#include "audio.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>		/* NULL */

#define PAGE_NUM 2

static volatile uint8_t audio_[PAGE_NUM][AUDIO_BUFFER_SIZE];
static volatile uint16_t counter_;
static volatile uint8_t stop_;
static volatile uint8_t current_page_;
static volatile uint8_t buffer_filled_;

void audio_init(void)
{
	AUDIO_OUTPUT_DDR |= (1 << AUDIO_OUTPUT_POS);

	counter_ = 0x00;
	stop_ = 0x01;
	current_page_ = 0xFF;
	buffer_filled_ = 0x00;

	TCCR0A = (1 << COM0A1) | (1 << WGM00);	/* clear OC0A on comp. match */
	TCCR0B = (1 << CS00) | (1 << WGM02);	/* no prescaler, phase correct pwm*/
	OCR0A = 128;

	sei();
}

void audio_intr(void)
{
	if(stop_)
	{
		OCR0A = 128;
	}
	else
	{
		/* next sample */
		OCR0A = audio_[current_page_][counter_];

		/* last data reached */
		if (++counter_ == AUDIO_BUFFER_SIZE)
		{
			counter_ = 0x00;

			if(!buffer_filled_)
				AUDIO_ERROR_FUNCTION()
			else
				current_page_ ^= 0x01;

			buffer_filled_ = 0x00;
		}
	}
}

volatile uint8_t* audio_get_buffer(uint8_t num)
{
	if(num < PAGE_NUM)
		return audio_[num];
	else
		return NULL;
}

volatile uint8_t* audio_get_free_buffer(void)
{
	if(!buffer_filled_)
	{
		if(current_page_ == 0xFF)
			return audio_[0];
		else
			return audio_[current_page_ ^ 0x01];
	}

	return NULL;
}

uint8_t audio_set_buffer(void)
{
	stop_ = 0x00;

	if(buffer_filled_)
		return 0x00;

	if(current_page_ != 0xFF)
		buffer_filled_ = 0x01;
	else
		current_page_ = 0x00;
/*	memcpy(audio_[current_page_ ^ 0x01], buffer); */

	return 0x01;
}

uint8_t audio_active(void)
{
	return !stop_;
}

void audio_stop(void)
{
	stop_ = 0x01;
}
